@@ -121,6 +121,25 @@ where
         self.inner.hash(key)
     }
 
+    pub(crate) fn contains_key<Q>(&self, key: &Q) -> bool
+    where
+        Arc<K>: Borrow<Q>,
+        Q: Hash + Eq + ?Sized,
+    {
+        match self.inner.get(key) {
+            None => false,
+            Some(entry) => {
+                let i = &self.inner;
+                let (ttl, tti, va) = (&i.time_to_live(), &i.time_to_idle(), &i.valid_after());
+                let now = i.current_time_from_expiration_clock();
+                let entry = &*entry;
+
+                !is_expired_entry_wo(ttl, va, entry, now)
+                    && !is_expired_entry_ao(tti, va, entry, now)
+            }
+        }
+    }
+
     pub(crate) fn get_with_hash<Q>(&self, key: &Q, hash: u64) -> Option<V>
     where
         Arc<K>: Borrow<Q>,
@@ -1318,10 +1337,15 @@ mod tests {
             ensure_sketch_len(pot16, pot16, "pot16");
             // due to ceiling to next_power_of_two
             ensure_sketch_len(pot16 + 1, pot(17), "pot16 + 1");
-            // due to ceiling to next_power_of_two
-            ensure_sketch_len(pot30 - 1, pot30, "pot30- 1");
-            ensure_sketch_len(pot30, pot30, "pot30");
-            ensure_sketch_len(u64::MAX, pot30, "u64::MAX");
+
+            // The following tests will allocate large memory (~8GiB).
+            // Skip when running on Circle CI.
+            if !cfg!(circleci) {
+                // due to ceiling to next_power_of_two
+                ensure_sketch_len(pot30 - 1, pot30, "pot30- 1");
+                ensure_sketch_len(pot30, pot30, "pot30");
+                ensure_sketch_len(u64::MAX, pot30, "u64::MAX");
+            }
         };
     }
 }
