@@ -1,4 +1,5 @@
-//! Provides a thread-safe, asynchronous (futures aware) cache implementation.
+//! Provides a thread-safe, concurrent asynchronous (futures aware) cache
+//! implementation.
 //!
 //! To use this module, enable a crate feature called "future".
 
@@ -7,7 +8,7 @@ mod cache;
 mod value_initializer;
 
 pub use builder::CacheBuilder;
-pub use cache::Cache;
+pub use cache::{BlockingOp, Cache};
 
 /// Provides extra methods that will be useful for testing.
 pub trait ConcurrentCacheExt<K, V> {
