@@ -362,6 +362,22 @@ where
         }
     }
 
+    /// Returns `true` if the cache contains a value for the key.
+    ///
+    /// Unlike the `get` method, this method is not considered a cache read operation,
+    /// so it does not update the historic popularity estimator or reset the idle
+    /// timer for the key.
+    ///
+    /// The key may be any borrowed form of the cache's key type, but `Hash` and `Eq`
+    /// on the borrowed form _must_ match those for the key type.
+    pub fn contains_key<Q>(&self, key: &Q) -> bool
+    where
+        Arc<K>: Borrow<Q>,
+        Q: Hash + Eq + ?Sized,
+    {
+        self.base.contains_key_with_hash(key, self.base.hash(key))
+    }
+
     /// Returns a _clone_ of the value corresponding to the key.
     ///
     /// If you want to store values that will be expensive to clone, wrap them by
@@ -920,17 +936,22 @@ mod tests {
         cache.insert("a", "alice").await;
         cache.insert("b", "bob").await;
         assert_eq!(cache.get(&"a"), Some("alice"));
+        assert!(cache.contains_key(&"a"));
+        assert!(cache.contains_key(&"b"));
         assert_eq!(cache.get(&"b"), Some("bob"));
         cache.sync();
         // counts: a -> 1, b -> 1
 
         cache.insert("c", "cindy").await;
         assert_eq!(cache.get(&"c"), Some("cindy"));
+        assert!(cache.contains_key(&"c"));
         // counts: a -> 1, b -> 1, c -> 1
         cache.sync();
 
+        assert!(cache.contains_key(&"a"));
         assert_eq!(cache.get(&"a"), Some("alice"));
         assert_eq!(cache.get(&"b"), Some("bob"));
+        assert!(cache.contains_key(&"b"));
         cache.sync();
         // counts: a -> 2, b -> 2, c -> 1
 
@@ -938,9 +959,11 @@ mod tests {
         cache.insert("d", "david").await; //   count: d -> 0
         cache.sync();
         assert_eq!(cache.get(&"d"), None); //   d -> 1
+        assert!(!cache.contains_key(&"d"));
 
         cache.insert("d", "david").await;
         cache.sync();
+        assert!(!cache.contains_key(&"d"));
         assert_eq!(cache.get(&"d"), None); //   d -> 2
 
         // "d" should be admitted and "c" should be evicted
@@ -951,9 +974,14 @@ mod tests {
         assert_eq!(cache.get(&"b"), Some("bob"));
         assert_eq!(cache.get(&"c"), None);
         assert_eq!(cache.get(&"d"), Some("dennis"));
+        assert!(cache.contains_key(&"a"));
+        assert!(cache.contains_key(&"b"));
+        assert!(!cache.contains_key(&"c"));
+        assert!(cache.contains_key(&"d"));
 
         cache.invalidate(&"b").await;
         assert_eq!(cache.get(&"b"), None);
+        assert!(!cache.contains_key(&"b"));
     }
 
     #[test]
@@ -1023,17 +1051,22 @@ mod tests {
         cache.insert("a", alice).await;
         cache.insert("b", bob).await;
         assert_eq!(cache.get(&"a"), Some(alice));
+        assert!(cache.contains_key(&"a"));
+        assert!(cache.contains_key(&"b"));
         assert_eq!(cache.get(&"b"), Some(bob));
         cache.sync();
         // order (LRU -> MRU) and counts: a -> 1, b -> 1
 
         cache.insert("c", cindy).await;
         assert_eq!(cache.get(&"c"), Some(cindy));
+        assert!(cache.contains_key(&"c"));
         // order and counts: a -> 1, b -> 1, c -> 1
         cache.sync();
 
+        assert!(cache.contains_key(&"a"));
         assert_eq!(cache.get(&"a"), Some(alice));
         assert_eq!(cache.get(&"b"), Some(bob));
+        assert!(cache.contains_key(&"b"));
         cache.sync();
         // order and counts: c -> 1, a -> 2, b -> 2
 
@@ -1043,17 +1076,21 @@ mod tests {
         cache.insert("d", david).await; //   count: d -> 0
         cache.sync();
         assert_eq!(cache.get(&"d"), None); //   d -> 1
+        assert!(!cache.contains_key(&"d"));
 
         cache.insert("d", david).await;
         cache.sync();
+        assert!(!cache.contains_key(&"d"));
         assert_eq!(cache.get(&"d"), None); //   d -> 2
 
         cache.insert("d", david).await;
         cache.sync();
         assert_eq!(cache.get(&"d"), None); //   d -> 3
+        assert!(!cache.contains_key(&"d"));
 
         cache.insert("d", david).await;
         cache.sync();
+        assert!(!cache.contains_key(&"d"));
         assert_eq!(cache.get(&"d"), None); //   d -> 4
 
         // Finally "d" should be admitted by evicting "c" and "a".
@@ -1063,12 +1100,18 @@ mod tests {
         assert_eq!(cache.get(&"b"), Some(bob));
         assert_eq!(cache.get(&"c"), None);
         assert_eq!(cache.get(&"d"), Some(dennis));
+        assert!(!cache.contains_key(&"a"));
+        assert!(cache.contains_key(&"b"));
+        assert!(!cache.contains_key(&"c"));
+        assert!(cache.contains_key(&"d"));
 
         // Update "b" with "bill" (w: 15 -> 20). This should evict "d" (w: 15).
         cache.insert("b", bill).await;
         cache.sync();
         assert_eq!(cache.get(&"b"), Some(bill));
         assert_eq!(cache.get(&"d"), None);
+        assert!(cache.contains_key(&"b"));
+        assert!(!cache.contains_key(&"d"));
 
         // Re-add "a" (w: 10) and update "b" with "bob" (w: 20 -> 15).
         cache.insert("a", alice).await;
@@ -1077,6 +1120,9 @@ mod tests {
         assert_eq!(cache.get(&"a"), Some(alice));
         assert_eq!(cache.get(&"b"), Some(bob));
         assert_eq!(cache.get(&"d"), None);
+        assert!(cache.contains_key(&"a"));
+        assert!(cache.contains_key(&"b"));
+        assert!(!cache.contains_key(&"d"));
 
         // Verify the sizes.
         assert_eq!(cache.estimated_entry_count(), 2);
@@ -1113,6 +1159,8 @@ mod tests {
 
         assert!(cache.get(&10).is_none());
         assert!(cache.get(&20).is_some());
+        assert!(!cache.contains_key(&10));
+        assert!(cache.contains_key(&20));
     }
 
     #[tokio::test]
@@ -1129,6 +1177,9 @@ mod tests {
         assert_eq!(cache.get(&"a"), Some("alice"));
         assert_eq!(cache.get(&"b"), Some("bob"));
         assert_eq!(cache.get(&"c"), Some("cindy"));
+        assert!(cache.contains_key(&"a"));
+        assert!(cache.contains_key(&"b"));
+        assert!(cache.contains_key(&"c"));
         cache.sync();
 
         cache.invalidate_all();
@@ -1141,6 +1192,10 @@ mod tests {
         assert!(cache.get(&"b").is_none());
         assert!(cache.get(&"c").is_none());
         assert_eq!(cache.get(&"d"), Some("david"));
+        assert!(!cache.contains_key(&"a"));
+        assert!(!cache.contains_key(&"b"));
+        assert!(!cache.contains_key(&"c"));
+        assert!(cache.contains_key(&"d"));
     }
 
     #[tokio::test]
@@ -1170,6 +1225,9 @@ mod tests {
         assert_eq!(cache.get(&0), Some("alice"));
         assert_eq!(cache.get(&1), Some("bob"));
         assert_eq!(cache.get(&2), Some("alex"));
+        assert!(cache.contains_key(&0));
+        assert!(cache.contains_key(&1));
+        assert!(cache.contains_key(&2));
 
         let names = ["alice", "alex"].iter().cloned().collect::<HashSet<_>>();
         cache.invalidate_entries_if(move |_k, &v| names.contains(v))?;
@@ -1190,6 +1248,12 @@ mod tests {
         assert_eq!(cache.get(&1), Some("bob"));
         // This should survive as it was inserted after calling invalidate_entries_if.
         assert_eq!(cache.get(&3), Some("alice"));
+
+        assert!(!cache.contains_key(&0));
+        assert!(cache.contains_key(&1));
+        assert!(!cache.contains_key(&2));
+        assert!(cache.contains_key(&3));
+
         assert_eq!(cache.estimated_entry_count(), 2);
         assert_eq!(cache.invalidation_predicate_count(), 0);
 
@@ -1207,6 +1271,10 @@ mod tests {
 
         assert!(cache.get(&1).is_none());
         assert!(cache.get(&3).is_none());
+
+        assert!(!cache.contains_key(&1));
+        assert!(!cache.contains_key(&3));
+
         assert_eq!(cache.estimated_entry_count(), 0);
         assert_eq!(cache.invalidation_predicate_count(), 0);
 
@@ -1234,12 +1302,14 @@ mod tests {
         mock.increment(Duration::from_secs(5)); // 5 secs from the start.
         cache.sync();
 
-        cache.get(&"a");
+        assert_eq!(cache.get(&"a"), Some("alice"));
+        assert!(cache.contains_key(&"a"));
 
         mock.increment(Duration::from_secs(5)); // 10 secs.
         cache.sync();
 
         assert_eq!(cache.get(&"a"), None);
+        assert!(!cache.contains_key(&"a"));
         assert!(cache.is_table_empty());
 
         cache.insert("b", "bob").await;
@@ -1251,6 +1321,7 @@ mod tests {
         cache.sync();
 
         assert_eq!(cache.get(&"b"), Some("bob"));
+        assert!(cache.contains_key(&"b"));
         assert_eq!(cache.estimated_entry_count(), 1);
 
         cache.insert("b", "bill").await;
@@ -1260,6 +1331,7 @@ mod tests {
         cache.sync();
 
         assert_eq!(cache.get(&"b"), Some("bill"));
+        assert!(cache.contains_key(&"b"));
         assert_eq!(cache.estimated_entry_count(), 1);
 
         mock.increment(Duration::from_secs(5)); // 25 secs
@@ -1267,6 +1339,8 @@ mod tests {
 
         assert_eq!(cache.get(&"a"), None);
         assert_eq!(cache.get(&"b"), None);
+        assert!(!cache.contains_key(&"a"));
+        assert!(!cache.contains_key(&"b"));
         assert!(cache.is_table_empty());
     }
 
@@ -1301,11 +1375,23 @@ mod tests {
 
         assert_eq!(cache.estimated_entry_count(), 2);
 
-        mock.increment(Duration::from_secs(5)); // 15 secs.
+        mock.increment(Duration::from_secs(2)); // 12 secs.
+        cache.sync();
+
+        // contains_key does not reset the idle timer for the key.
+        assert!(cache.contains_key(&"a"));
+        assert!(cache.contains_key(&"b"));
+        cache.sync();
+
+        assert_eq!(cache.estimated_entry_count(), 2);
+
+        mock.increment(Duration::from_secs(3)); // 15 secs.
         cache.sync();
 
         assert_eq!(cache.get(&"a"), None);
         assert_eq!(cache.get(&"b"), Some("bob"));
+        assert!(!cache.contains_key(&"a"));
+        assert!(cache.contains_key(&"b"));
         assert_eq!(cache.estimated_entry_count(), 1);
 
         mock.increment(Duration::from_secs(10)); // 25 secs
@@ -1313,6 +1399,8 @@ mod tests {
 
         assert_eq!(cache.get(&"a"), None);
         assert_eq!(cache.get(&"b"), None);
+        assert!(!cache.contains_key(&"a"));
+        assert!(!cache.contains_key(&"b"));
         assert!(cache.is_table_empty());
     }
 
