@@ -237,6 +237,35 @@ where
         }
     }
 
+    /// Returns `true` if the cache contains a value for the key.
+    ///
+    /// Unlike the `get` method, this method is not considered a cache read operation,
+    /// so it does not update the historic popularity estimator or reset the idle
+    /// timer for the key.
+    ///
+    /// The key may be any borrowed form of the cache's key type, but `Hash` and `Eq`
+    /// on the borrowed form _must_ match those for the key type.
+    pub fn contains_key<Q>(&mut self, key: &Q) -> bool
+    where
+        Rc<K>: Borrow<Q>,
+        Q: Hash + Eq + ?Sized,
+    {
+        let timestamp = self.evict_expired_if_needed();
+        self.evict_lru_entries();
+
+        match (self.cache.get(key), timestamp) {
+            // Value not found.
+            (None, _) => false,
+            // Value found, no expiry.
+            (Some(_), None) => true,
+            // Value found, check if expired.
+            (Some(entry), Some(ts)) => {
+                !Self::is_expired_entry_wo(&self.time_to_live, entry, ts)
+                    && !Self::is_expired_entry_ao(&self.time_to_idle, entry, ts)
+            }
+        }
+    }
+
     /// Returns an immutable reference of the value corresponding to the key.
     ///
     /// The key may be any borrowed form of the cache's key type, but `Hash` and `Eq`
@@ -940,22 +969,29 @@ mod tests {
         cache.insert("a", "alice");
         cache.insert("b", "bob");
         assert_eq!(cache.get(&"a"), Some(&"alice"));
+        assert!(cache.contains_key(&"a"));
+        assert!(cache.contains_key(&"b"));
         assert_eq!(cache.get(&"b"), Some(&"bob"));
         // counts: a -> 1, b -> 1
 
         cache.insert("c", "cindy");
         assert_eq!(cache.get(&"c"), Some(&"cindy"));
+        assert!(cache.contains_key(&"c"));
         // counts: a -> 1, b -> 1, c -> 1
 
+        assert!(cache.contains_key(&"a"));
         assert_eq!(cache.get(&"a"), Some(&"alice"));
         assert_eq!(cache.get(&"b"), Some(&"bob"));
+        assert!(cache.contains_key(&"b"));
         // counts: a -> 2, b -> 2, c -> 1
 
         // "d" should not be admitted because its frequency is too low.
         cache.insert("d", "david"); //   count: d -> 0
         assert_eq!(cache.get(&"d"), None); //   d -> 1
+        assert!(!cache.contains_key(&"d"));
 
         cache.insert("d", "david");
+        assert!(!cache.contains_key(&"d"));
         assert_eq!(cache.get(&"d"), None); //   d -> 2
 
         // "d" should be admitted and "c" should be evicted
@@ -965,9 +1001,14 @@ mod tests {
         assert_eq!(cache.get(&"b"), Some(&"bob"));
         assert_eq!(cache.get(&"c"), None);
         assert_eq!(cache.get(&"d"), Some(&"dennis"));
+        assert!(cache.contains_key(&"a"));
+        assert!(cache.contains_key(&"b"));
+        assert!(!cache.contains_key(&"c"));
+        assert!(cache.contains_key(&"d"));
 
         cache.invalidate(&"b");
         assert_eq!(cache.get(&"b"), None);
+        assert!(!cache.contains_key(&"b"));
     }
 
     #[test]
@@ -987,15 +1028,20 @@ mod tests {
         cache.insert("a", alice);
         cache.insert("b", bob);
         assert_eq!(cache.get(&"a"), Some(&alice));
+        assert!(cache.contains_key(&"a"));
+        assert!(cache.contains_key(&"b"));
         assert_eq!(cache.get(&"b"), Some(&bob));
         // order (LRU -> MRU) and counts: a -> 1, b -> 1
 
         cache.insert("c", cindy);
         assert_eq!(cache.get(&"c"), Some(&cindy));
+        assert!(cache.contains_key(&"c"));
         // order and counts: a -> 1, b -> 1, c -> 1
 
+        assert!(cache.contains_key(&"a"));
         assert_eq!(cache.get(&"a"), Some(&alice));
         assert_eq!(cache.get(&"b"), Some(&bob));
+        assert!(cache.contains_key(&"b"));
         // order and counts: c -> 1, a -> 2, b -> 2
 
         // To enter "d" (weight: 15), it needs to evict "c" (w: 5) and "a" (w: 10).
@@ -1003,14 +1049,18 @@ mod tests {
         // of "a" and "c".
         cache.insert("d", david); //   count: d -> 0
         assert_eq!(cache.get(&"d"), None); //   d -> 1
+        assert!(!cache.contains_key(&"d"));
 
         cache.insert("d", david);
+        assert!(!cache.contains_key(&"d"));
         assert_eq!(cache.get(&"d"), None); //   d -> 2
 
         cache.insert("d", david);
         assert_eq!(cache.get(&"d"), None); //   d -> 3
+        assert!(!cache.contains_key(&"d"));
 
         cache.insert("d", david);
+        assert!(!cache.contains_key(&"d"));
         assert_eq!(cache.get(&"d"), None); //   d -> 4
 
         // Finally "d" should be admitted by evicting "c" and "a".
@@ -1019,11 +1069,17 @@ mod tests {
         assert_eq!(cache.get(&"b"), Some(&bob));
         assert_eq!(cache.get(&"c"), None);
         assert_eq!(cache.get(&"d"), Some(&dennis));
+        assert!(!cache.contains_key(&"a"));
+        assert!(cache.contains_key(&"b"));
+        assert!(!cache.contains_key(&"c"));
+        assert!(cache.contains_key(&"d"));
 
         // Update "b" with "bill" (w: 15 -> 20). This should evict "d" (w: 15).
         cache.insert("b", bill);
         assert_eq!(cache.get(&"b"), Some(&bill));
         assert_eq!(cache.get(&"d"), None);
+        assert!(cache.contains_key(&"b"));
+        assert!(!cache.contains_key(&"d"));
 
         // Re-add "a" (w: 10) and update "b" with "bob" (w: 20 -> 15).
         cache.insert("a", alice);
@@ -1031,6 +1087,9 @@ mod tests {
         assert_eq!(cache.get(&"a"), Some(&alice));
         assert_eq!(cache.get(&"b"), Some(&bob));
         assert_eq!(cache.get(&"d"), None);
+        assert!(cache.contains_key(&"a"));
+        assert!(cache.contains_key(&"b"));
+        assert!(!cache.contains_key(&"d"));
 
         // Verify the sizes.
         assert_eq!(cache.entry_count, 2);
@@ -1048,6 +1107,9 @@ mod tests {
         assert_eq!(cache.get(&"a"), Some(&"alice"));
         assert_eq!(cache.get(&"b"), Some(&"bob"));
         assert_eq!(cache.get(&"c"), Some(&"cindy"));
+        assert!(cache.contains_key(&"a"));
+        assert!(cache.contains_key(&"b"));
+        assert!(cache.contains_key(&"c"));
 
         cache.invalidate_all();
 
@@ -1057,6 +1119,10 @@ mod tests {
         assert!(cache.get(&"b").is_none());
         assert!(cache.get(&"c").is_none());
         assert_eq!(cache.get(&"d"), Some(&"david"));
+        assert!(!cache.contains_key(&"a"));
+        assert!(!cache.contains_key(&"b"));
+        assert!(!cache.contains_key(&"c"));
+        assert!(cache.contains_key(&"d"));
     }
 
     #[test]
@@ -1078,6 +1144,9 @@ mod tests {
         assert_eq!(cache.get(&0), Some(&"alice"));
         assert_eq!(cache.get(&1), Some(&"bob"));
         assert_eq!(cache.get(&2), Some(&"alex"));
+        assert!(cache.contains_key(&0));
+        assert!(cache.contains_key(&1));
+        assert!(cache.contains_key(&2));
 
         let names = ["alice", "alex"].iter().cloned().collect::<HashSet<_>>();
         cache.invalidate_entries_if(move |_k, &v| names.contains(v));
@@ -1091,6 +1160,12 @@ mod tests {
         assert_eq!(cache.get(&1), Some(&"bob"));
         // This should survive as it was inserted after calling invalidate_entries_if.
         assert_eq!(cache.get(&3), Some(&"alice"));
+
+        assert!(!cache.contains_key(&0));
+        assert!(cache.contains_key(&1));
+        assert!(!cache.contains_key(&2));
+        assert!(cache.contains_key(&3));
+
         assert_eq!(cache.cache.len(), 2);
 
         mock.increment(Duration::from_secs(5)); // 15 secs from the start.
@@ -1100,6 +1175,10 @@ mod tests {
 
         assert!(cache.get(&1).is_none());
         assert!(cache.get(&3).is_none());
+
+        assert!(!cache.contains_key(&1));
+        assert!(!cache.contains_key(&3));
+
         assert_eq!(cache.cache.len(), 0);
     }
 
@@ -1118,11 +1197,13 @@ mod tests {
 
         mock.increment(Duration::from_secs(5)); // 5 secs from the start.
 
-        cache.get(&"a");
+        assert_eq!(cache.get(&"a"), Some(&"alice"));
+        assert!(cache.contains_key(&"a"));
 
         mock.increment(Duration::from_secs(5)); // 10 secs.
 
         assert_eq!(cache.get(&"a"), None);
+        assert!(!cache.contains_key(&"a"));
         assert!(cache.cache.is_empty());
 
         cache.insert("b", "bob");
@@ -1132,6 +1213,7 @@ mod tests {
         mock.increment(Duration::from_secs(5)); // 15 secs.
 
         assert_eq!(cache.get(&"b"), Some(&"bob"));
+        assert!(cache.contains_key(&"b"));
         assert_eq!(cache.cache.len(), 1);
 
         cache.insert("b", "bill");
@@ -1139,12 +1221,15 @@ mod tests {
         mock.increment(Duration::from_secs(5)); // 20 secs
 
         assert_eq!(cache.get(&"b"), Some(&"bill"));
+        assert!(cache.contains_key(&"b"));
         assert_eq!(cache.cache.len(), 1);
 
         mock.increment(Duration::from_secs(5)); // 25 secs
 
         assert_eq!(cache.get(&"a"), None);
         assert_eq!(cache.get(&"b"), None);
+        assert!(!cache.contains_key(&"a"));
+        assert!(!cache.contains_key(&"b"));
         assert!(cache.cache.is_empty());
     }
 
@@ -1171,16 +1256,28 @@ mod tests {
 
         assert_eq!(cache.cache.len(), 2);
 
-        mock.increment(Duration::from_secs(5)); // 15 secs.
+        mock.increment(Duration::from_secs(2)); // 12 secs.
+
+        // contains_key does not reset the idle timer for the key.
+        assert!(cache.contains_key(&"a"));
+        assert!(cache.contains_key(&"b"));
+
+        assert_eq!(cache.cache.len(), 2);
+
+        mock.increment(Duration::from_secs(3)); // 15 secs.
 
         assert_eq!(cache.get(&"a"), None);
         assert_eq!(cache.get(&"b"), Some(&"bob"));
+        assert!(!cache.contains_key(&"a"));
+        assert!(cache.contains_key(&"b"));
         assert_eq!(cache.cache.len(), 1);
 
         mock.increment(Duration::from_secs(10)); // 25 secs
 
         assert_eq!(cache.get(&"a"), None);
         assert_eq!(cache.get(&"b"), None);
+        assert!(!cache.contains_key(&"a"));
+        assert!(!cache.contains_key(&"b"));
         assert!(cache.cache.is_empty());
     }
 
@@ -1218,10 +1315,15 @@ mod tests {
             ensure_sketch_len(pot16, pot16, "pot16");
             // due to ceiling to next_power_of_two
             ensure_sketch_len(pot16 + 1, pot(17), "pot16 + 1");
-            // due to ceiling to next_power_of_two
-            ensure_sketch_len(pot30 - 1, pot30, "pot30- 1");
-            ensure_sketch_len(pot30, pot30, "pot30");
-            ensure_sketch_len(u64::MAX, pot30, "u64::MAX");
+
+            // The following tests will allocate large memory (~8GiB).
+            // Skip when running on Circle CI.
+            if !cfg!(circleci) {
+                // due to ceiling to next_power_of_two
+                ensure_sketch_len(pot30 - 1, pot30, "pot30- 1");
+                ensure_sketch_len(pot30, pot30, "pot30");
+                ensure_sketch_len(u64::MAX, pot30, "u64::MAX");
+            }
         };
     }
 }
